-------------------------------------------------------------------------------
-- Title      : Top-level entity for implementation on SVEC carrier card
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : svec_top_ffpg.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
-- Company    : CERN (BE-BI-QP)
-- Created    : 2016-07-26
-- Last update: 2023-02-23
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Contains code for two FMC mezzanines, communication via VME bus
--              and other board specific code.
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author          Comment
-- 2016-08-24  1.0      Jan Pospisil
-- 2023-02-23  2.0      Tom Levens      Port to new SVEC reference design
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.wr_board_pkg.all;
use work.wr_fabric_pkg.all;
use work.FfpgPkg.all;
use work.sourceid_svec_top_ffpg_pkg;

library unisim;
use unisim.vcomponents.all;

entity svec_top_ffpg is
  generic (
    -- Simulation-mode enable parameter. Set by default (synthesis) to 0, and
    -- changed to non-zero in the instantiation of the top level DUT in the testbench.
    -- Its purpose is to reduce some internal counters/timeouts to speed up simulations.
    g_SIMULATION : integer := 0;
    -- Increase information messages during simulation
    g_VERBOSE    : boolean := False
  );
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Reset from system fpga
    rst_n_i : in std_logic;

    -- 125 MHz PLL reference
    clk_125m_pllref_p_i : in std_logic;
    clk_125m_pllref_n_i : in std_logic;

    -- 20MHz VCXO clock (for WR)
    clk_20m_vcxo_i : in std_logic := '0';

    -- 125 MHz GTP reference
    clk_125m_gtp_n_i : in std_logic := '0';
    clk_125m_gtp_p_i : in std_logic := '0';

    ---------------------------------------------------------------------------
    -- VME interface
    ---------------------------------------------------------------------------

    vme_write_n_i    : in    std_logic;
    vme_sysreset_n_i : in    std_logic;
    vme_retry_oe_o   : out   std_logic;
    vme_retry_n_o    : out   std_logic;
    vme_lword_n_b    : inout std_logic;
    vme_iackout_n_o  : out   std_logic;
    vme_iackin_n_i   : in    std_logic;
    vme_iack_n_i     : in    std_logic;
    vme_gap_i        : in    std_logic;
    vme_dtack_oe_o   : out   std_logic;
    vme_dtack_n_o    : out   std_logic;
    vme_ds_n_i       : in    std_logic_vector(1 downto 0);
    vme_data_oe_n_o  : out   std_logic;
    vme_data_dir_o   : out   std_logic;
    vme_berr_o       : out   std_logic;
    vme_as_n_i       : in    std_logic;
    vme_addr_oe_n_o  : out   std_logic;
    vme_addr_dir_o   : out   std_logic;
    vme_irq_o        : out   std_logic_vector(7 downto 1);
    vme_ga_i         : in    std_logic_vector(4 downto 0);
    vme_data_b       : inout std_logic_vector(31 downto 0);
    vme_am_i         : in    std_logic_vector(5 downto 0);
    vme_addr_b       : inout std_logic_vector(31 downto 1);

    ---------------------------------------------------------------------------
    -- FMC interface
    ---------------------------------------------------------------------------

    -- I2C interface for accessing FMC EEPROM.
    fmc0_scl_b : inout std_logic;
    fmc0_sda_b : inout std_logic;
    fmc1_scl_b : inout std_logic;
    fmc1_sda_b : inout std_logic;

    -- Presence  (there is a pull-up)
    fmc0_prsnt_m2c_n_i: in std_logic;
    fmc1_prsnt_m2c_n_i: in std_logic;

    ---------------------------------------------------------------------------
    -- Carrier
    ---------------------------------------------------------------------------

    -- Onewire interface
    onewire_b : inout std_logic;

    -- Carrier I2C eeprom
    carrier_scl_b : inout std_logic;
    carrier_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- Flash memory SPI interface
    ---------------------------------------------------------------------------

    spi_sclk_o : out std_logic;
    spi_ncs_o  : out std_logic;
    spi_mosi_o : out std_logic;
    spi_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic := '1';
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI interface to DACs
    ---------------------------------------------------------------------------

    plldac_sclk_o     : out std_logic;
    plldac_din_o      : out std_logic;
    pll20dac_din_o    : out std_logic;
    pll20dac_sclk_o   : out std_logic;
    pll20dac_sync_n_o : out std_logic;
    pll25dac_din_o    : out std_logic;
    pll25dac_sclk_o   : out std_logic;
    pll25dac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic := '0';
    sfp_rxn_i         : in    std_logic := '0';
    sfp_mod_def0_i    : in    std_logic := '0';   -- sfp detect
    sfp_mod_def1_b    : inout std_logic;          -- scl
    sfp_mod_def2_b    : inout std_logic;          -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic := '0';
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic := '0';

    ---------------------------------------------------------------------------
    -- FMC slot 0
    ---------------------------------------------------------------------------

    -- clock
    Fmc0ClkIn0P_ik: in std_logic;
    Fmc0ClkIn0N_ik: in std_logic;
    -- DACs
    Fmc0TriggerDac_o: out t_Ad5600Interface;
    Fmc0VcxoDac_o: out t_Ad5600Interface;
    -- output enable
    Fmc0Ch1OutputEnable_o: out std_logic;
    Fmc0Ch2OutputEnable_o: out std_logic;
    -- signal feedback
    Fmc0Ch1OutCal_i: in std_logic;
    Fmc0Ch2OutCal_i: in std_logic;
    -- delay configuration
    Fmc0DelayValue_ob: out unsigned(9 downto 0);
    Fmc0Ch1SetLe_on: out std_logic;
    Fmc0Ch1ResLe_on: out std_logic;
    Fmc0Ch2SetLe_on: out std_logic;
    Fmc0Ch2ResLe_on: out std_logic;
    Fmc0Ch1SetP_o: out std_logic;
    Fmc0Ch1SetN_o: out std_logic;
    Fmc0Ch1ResP_o: out std_logic;
    Fmc0Ch1ResN_o: out std_logic;
    Fmc0Ch2SetP_o: out std_logic;
    Fmc0Ch2SetN_o: out std_logic;
    Fmc0Ch2ResP_o: out std_logic;
    Fmc0Ch2ResN_o: out std_logic;
    Fmc0TriggerP_i: in std_logic;
    Fmc0TriggerN_i: in std_logic;
    -- AD9512 SPI
    Fmc0SpiAd9512Sclk_o: out std_logic;
    Fmc0SpiAd9512Mosi_o: out std_logic;
    Fmc0SpiAd9512Miso_i: in std_logic;
    Fmc0SpiAd9512Cs_on: out std_logic;
    -- AD9512 func. pin
    Fmc0Ad9512Func_o: out std_logic;
    -- clock selection pin of SY58017U
    Fmc0Clk2Sel_o: out std_logic;
    -- clock output
    Fmc0ClkOutP_ok: out std_logic;
    Fmc0ClkOutN_ok: out std_logic;
    -- temperature chip interface
    Fmc0Onewire_io: inout std_logic;
    -- LEDs
    Fmc0Led_ob: out std_logic_vector(4 downto 1);

    ---------------------------------------------------------------------------
    -- FMC slot 1
    ---------------------------------------------------------------------------

    -- clock
    Fmc1ClkIn0P_ik: in std_logic;
    Fmc1ClkIn0N_ik: in std_logic;
    -- DACs
    Fmc1TriggerDac_o: out t_Ad5600Interface;
    Fmc1VcxoDac_o: out t_Ad5600Interface;
    -- output enable
    Fmc1Ch1OutputEnable_o: out std_logic;
    Fmc1Ch2OutputEnable_o: out std_logic;
    -- signal feedback
    Fmc1Ch1OutCal_i: in std_logic;
    Fmc1Ch2OutCal_i: in std_logic;
    -- delay configuration
    Fmc1DelayValue_ob: out unsigned(9 downto 0);
    Fmc1Ch1SetLe_on: out std_logic;
    Fmc1Ch1ResLe_on: out std_logic;
    Fmc1Ch2SetLe_on: out std_logic;
    Fmc1Ch2ResLe_on: out std_logic;
    Fmc1Ch1SetP_o: out std_logic;
    Fmc1Ch1SetN_o: out std_logic;
    Fmc1Ch1ResP_o: out std_logic;
    Fmc1Ch1ResN_o: out std_logic;
    Fmc1Ch2SetP_o: out std_logic;
    Fmc1Ch2SetN_o: out std_logic;
    Fmc1Ch2ResP_o: out std_logic;
    Fmc1Ch2ResN_o: out std_logic;
    Fmc1TriggerP_i: in std_logic;
    Fmc1TriggerN_i: in std_logic;
    -- AD9512 SPI
    Fmc1SpiAd9512Sclk_o: out std_logic;
    Fmc1SpiAd9512Mosi_o: out std_logic;
    Fmc1SpiAd9512Miso_i: in std_logic;
    Fmc1SpiAd9512Cs_on: out std_logic;
    -- AD9512 func. pin
    Fmc1Ad9512Func_o: out std_logic;
    -- clock selection pin of SY58017U
    Fmc1Clk2Sel_o: out std_logic;
    -- clock output
    Fmc1ClkOutP_ok: out std_logic;
    Fmc1ClkOutN_ok: out std_logic;
    -- temperature chip interface
    Fmc1Onewire_io: inout std_logic;
    -- LEDs
    Fmc1Led_ob: out std_logic_vector(4 downto 1);

    ---------------------------------------------------------------------------
    -- General
    ---------------------------------------------------------------------------

    -- PCB revision
    pcbrev_i : in std_logic_vector(4 downto 0);

    -- Carrier front panel LEDs
    fp_led_line_oen_o : out std_logic_vector(1 downto 0);
    fp_led_line_o     : out std_logic_vector(1 downto 0);
    fp_led_column_o   : out std_logic_vector(3 downto 0);

    -- GPIO
    fp_gpio1_b      : in  std_logic;
    fp_gpio2_b      : in  std_logic;
    fp_gpio3_b      : in  std_logic;
    fp_gpio4_b      : in  std_logic;
    fp_term_en_o    : out std_logic_vector(4 downto 1);
    fp_gpio1_a2b_o  : out std_logic;
    fp_gpio2_a2b_o  : out std_logic;
    fp_gpio34_a2b_o : out std_logic
  );
end entity svec_top_ffpg;

architecture top of svec_top_ffpg is

  -- Clocks & reset
  signal clk_sys_62m5     : std_logic;
  signal rst_sys_62m5_n   : std_logic;

  -- Wishbone
  signal master_wb_out    : t_wishbone_master_out;
  signal master_wb_in     : t_wishbone_master_in;

  signal metadata_wb_out, ffpg0_wb_out, ffpg1_wb_out : t_wishbone_slave_out;
  signal metadata_wb_in,  ffpg0_wb_in,  ffpg1_wb_in  : t_wishbone_slave_in;

  -- For differential inputs
  signal Fmc0ClkIn0_k, Fmc0Trigger  : std_logic;
  signal Fmc1ClkIn0_k, Fmc1Trigger  : std_logic;

  -- For differentials outputs
  signal Fmc0Ch1Set, Fmc0Ch1Res, Fmc0Ch2Set, Fmc0Ch2Res : std_logic;
  signal Fmc0ClkOutDdr, Fmc0ClkOut_k                    : std_logic;
  signal Fmc1Ch1Set, Fmc1Ch1Res, Fmc1Ch2Set, Fmc1Ch2Res : std_logic;
  signal Fmc1ClkOutDdr, Fmc1ClkOut_k                    : std_logic;

  -- Front panel LED control
  signal led_state      : std_logic_vector(15 downto 0);
  signal vme_access     : std_logic;

  signal fp_gpio_in     : std_logic_vector(4 downto 1);
  signal fp_gpio        : std_logic_vector(4 downto 1);
  signal fp_gpio_led    : std_logic_vector(4 downto 1);

begin

  cmp_svec_top_ffpg_map : entity work.svec_top_ffpg_map
    port map (
      rst_n_i         => rst_sys_62m5_n,
      clk_i           => clk_sys_62m5,
      wb_i            => master_wb_out,
      wb_o            => master_wb_in,
      metadata_i      => metadata_wb_out,
      metadata_o      => metadata_wb_in,
      ffpg0_i         => ffpg0_wb_out,
      ffpg0_o         => ffpg0_wb_in,
      ffpg1_i         => ffpg1_wb_out,
      ffpg1_o         => ffpg1_wb_in
    );

  cmp_xwb_metadata : entity work.xwb_metadata
    generic map (
      g_VENDOR_ID     => x"0000_10DC",
      g_DEVICE_ID     => x"465047_01",  -- FPG_01,
      g_VERSION       => sourceid_svec_top_ffpg_pkg.version,
      g_CAPABILITIES  => x"0000_0000",
      g_COMMIT_ID     => sourceid_svec_top_ffpg_pkg.sourceid
    )
    port map (
      clk_i           => clk_sys_62m5,
      rst_n_i         => rst_sys_62m5_n,
      wb_i            => metadata_wb_in,
      wb_o            => metadata_wb_out
    );

  inst_svec_base: entity work.svec_base_wr
    generic map (
      g_with_vic            => False,
      g_with_onewire        => True,
      g_with_spi            => True,
      g_with_wr             => False,
      g_with_ddr4           => False,
      g_with_ddr5           => False,
      g_app_offset          => x"0000_4000",
      g_num_user_irq        => 0,
      g_dpram_initf         => "",
      g_fabric_iface        => open,
      g_streamers_op_mode   => open,
      g_tx_streamer_params  => open,
      g_rx_streamer_params  => open,
      g_simulation          => g_SIMULATION,
      g_verbose             => g_VERBOSE
    )
    port map (
      rst_n_i               => rst_n_i,
      clk_125m_pllref_p_i   => clk_125m_pllref_p_i,
      clk_125m_pllref_n_i   => clk_125m_pllref_n_i,
      clk_20m_vcxo_i        => clk_20m_vcxo_i,
      clk_125m_gtp_n_i      => clk_125m_gtp_n_i,
      clk_125m_gtp_p_i      => clk_125m_gtp_p_i,
      vme_write_n_i         => vme_write_n_i,
      vme_sysreset_n_i      => vme_sysreset_n_i,
      vme_retry_oe_o        => vme_retry_oe_o,
      vme_retry_n_o         => vme_retry_n_o,
      vme_lword_n_b         => vme_lword_n_b,
      vme_iackout_n_o       => vme_iackout_n_o,
      vme_iackin_n_i        => vme_iackin_n_i,
      vme_iack_n_i          => vme_iack_n_i,
      vme_gap_i             => vme_gap_i,
      vme_dtack_oe_o        => vme_dtack_oe_o,
      vme_dtack_n_o         => vme_dtack_n_o,
      vme_ds_n_i            => vme_ds_n_i,
      vme_data_oe_n_o       => vme_data_oe_n_o,
      vme_data_dir_o        => vme_data_dir_o,
      vme_berr_o            => vme_berr_o,
      vme_as_n_i            => vme_as_n_i,
      vme_addr_oe_n_o       => vme_addr_oe_n_o,
      vme_addr_dir_o        => vme_addr_dir_o,
      vme_irq_o             => vme_irq_o,
      vme_ga_i              => vme_ga_i,
      vme_data_b            => vme_data_b,
      vme_am_i              => vme_am_i,
      vme_addr_b            => vme_addr_b,
      fmc0_scl_b            => fmc0_scl_b,
      fmc0_sda_b            => fmc0_sda_b,
      fmc1_scl_b            => fmc1_scl_b,
      fmc1_sda_b            => fmc1_sda_b,
      fmc0_prsnt_m2c_n_i    => fmc0_prsnt_m2c_n_i,
      fmc1_prsnt_m2c_n_i    => fmc1_prsnt_m2c_n_i,
      onewire_b             => onewire_b,
      carrier_scl_b         => carrier_scl_b,
      carrier_sda_b         => carrier_sda_b,
      spi_sclk_o            => spi_sclk_o,
      spi_ncs_o             => spi_ncs_o,
      spi_mosi_o            => spi_mosi_o,
      spi_miso_i            => spi_miso_i,
      uart_rxd_i            => uart_rxd_i,
      uart_txd_o            => uart_txd_o,
      plldac_sclk_o         => plldac_sclk_o,
      plldac_din_o          => plldac_din_o,
      pll20dac_din_o        => pll20dac_din_o,
      pll20dac_sclk_o       => pll20dac_sclk_o,
      pll20dac_sync_n_o     => pll20dac_sync_n_o,
      pll25dac_din_o        => pll25dac_din_o,
      pll25dac_sclk_o       => pll25dac_sclk_o,
      pll25dac_sync_n_o     => pll25dac_sync_n_o,
      sfp_txp_o             => sfp_txp_o,
      sfp_txn_o             => sfp_txn_o,
      sfp_rxp_i             => sfp_rxp_i,
      sfp_rxn_i             => sfp_rxn_i,
      sfp_mod_def0_i        => sfp_mod_def0_i,
      sfp_mod_def1_b        => sfp_mod_def1_b,
      sfp_mod_def2_b        => sfp_mod_def2_b,
      sfp_rate_select_o     => sfp_rate_select_o,
      sfp_tx_fault_i        => sfp_tx_fault_i,
      sfp_tx_disable_o      => sfp_tx_disable_o,
      sfp_los_i             => sfp_los_i,
      ddr4_a_o              => open,
      ddr4_ba_o             => open,
      ddr4_cas_n_o          => open,
      ddr4_ck_n_o           => open,
      ddr4_ck_p_o           => open,
      ddr4_cke_o            => open,
      ddr4_dq_b             => open,
      ddr4_ldm_o            => open,
      ddr4_ldqs_n_b         => open,
      ddr4_ldqs_p_b         => open,
      ddr4_odt_o            => open,
      ddr4_ras_n_o          => open,
      ddr4_reset_n_o        => open,
      ddr4_rzq_b            => open,
      ddr4_udm_o            => open,
      ddr4_udqs_n_b         => open,
      ddr4_udqs_p_b         => open,
      ddr4_we_n_o           => open,
      ddr5_a_o              => open,
      ddr5_ba_o             => open,
      ddr5_cas_n_o          => open,
      ddr5_ck_n_o           => open,
      ddr5_ck_p_o           => open,
      ddr5_cke_o            => open,
      ddr5_dq_b             => open,
      ddr5_ldm_o            => open,
      ddr5_ldqs_n_b         => open,
      ddr5_ldqs_p_b         => open,
      ddr5_odt_o            => open,
      ddr5_ras_n_o          => open,
      ddr5_reset_n_o        => open,
      ddr5_rzq_b            => open,
      ddr5_udm_o            => open,
      ddr5_udqs_n_b         => open,
      ddr5_udqs_p_b         => open,
      ddr5_we_n_o           => open,
      pcbrev_i              => pcbrev_i,
      ddr4_clk_i            => clk_sys_62m5,
      ddr4_rst_n_i          => rst_sys_62m5_n,
      ddr4_wb_i.cyc         => '0',
      ddr4_wb_i.stb         => '0',
      ddr4_wb_i.adr         => x"0000_0000",
      ddr4_wb_i.sel         => x"00",
      ddr4_wb_i.we          => '0',
      ddr4_wb_i.dat         => (63 downto 0 => '0'),
      ddr4_wb_o             => open,
      ddr5_clk_i            => clk_sys_62m5,
      ddr5_rst_n_i          => rst_sys_62m5_n,
      ddr5_wb_i.cyc         => '0',
      ddr5_wb_i.stb         => '0',
      ddr5_wb_i.adr         => x"0000_0000",
      ddr5_wb_i.sel         => x"00",
      ddr5_wb_i.we          => '0',
      ddr5_wb_i.dat         => (63 downto 0 => '0'),
      ddr5_wb_o             => open,
      ddr4_wr_fifo_empty_o  => open,
      ddr5_wr_fifo_empty_o  => open,
      clk_sys_62m5_o        => clk_sys_62m5,
      rst_sys_62m5_n_o      => rst_sys_62m5_n,
      clk_ref_125m_o        => open,
      rst_ref_125m_n_o      => open,
      irq_user_i            => "",
      wrf_src_o             => open,
      wrf_src_i             => open,
      wrf_snk_o             => open,
      wrf_snk_i             => open,
      wrs_tx_data_i         => open,
      wrs_tx_valid_i        => open,
      wrs_tx_dreq_o         => open,
      wrs_tx_last_i         => open,
      wrs_tx_flush_i        => open,
      wrs_tx_cfg_i          => open,
      wrs_rx_first_o        => open,
      wrs_rx_last_o         => open,
      wrs_rx_data_o         => open,
      wrs_rx_valid_o        => open,
      wrs_rx_dreq_i         => open,
      wrs_rx_cfg_i          => open,
      wb_eth_master_o       => open,
      wb_eth_master_i       => open,
      tm_link_up_o          => open,
      tm_time_valid_o       => open,
      tm_tai_o              => open,
      tm_cycles_o           => open,
      pps_p_o               => open,
      pps_led_o             => open,
      link_ok_o             => open,
      led_link_o            => open,
      led_act_o             => open,
      app_wb_o              => master_wb_out,
      app_wb_i              => master_wb_in
    );

  ------------------------------------------------------------------------------
  -- Carrier front panel LEDs and LEMOs
  ------------------------------------------------------------------------------

  cmp_led_controller : entity work.gc_bicolor_led_ctrl
    generic map (
      g_nb_column     => 4,
      g_nb_line       => 2,
      g_clk_freq      => 62500000,    -- in Hz
      g_refresh_rate  => 250          -- in Hz
    )
    port map (
      rst_n_i         => rst_sys_62m5_n,
      clk_i           => clk_sys_62m5,
      led_intensity_i => "1100100",  -- in %
      led_state_i     => led_state,
      column_o        => fp_led_column_o,
      line_o          => fp_led_line_o,
      line_oen_o      => fp_led_line_oen_o
    );

  cmp_vme_access_led : gc_extend_pulse
    generic map (
      g_width     => 2500000
    )
    port map (
      clk_i       => clk_sys_62m5,
      rst_n_i     => rst_sys_62m5_n,
      pulse_i     => master_wb_out.stb,
      extended_o  => vme_access
    );

  -- LED order on front panel (top to bottom)
  --  7-6    5-4    3-2   1-0
  -- 15-14  13-12  11-10  9-8
  led_state(1 downto 0)   <= c_led_green when vme_access = '1' else c_led_off;
  led_state(3 downto 2)   <= c_led_off;
  led_state(5 downto 4)   <= c_led_off;
  led_state(7 downto 6)   <= c_led_off;
  led_state(9 downto 8)   <= c_led_green when fp_gpio_led(4) = '1' else c_led_off;
  led_state(11 downto 10) <= c_led_green when fp_gpio_led(3) = '1' else c_led_off;
  led_state(13 downto 12) <= c_led_green when fp_gpio_led(2) = '1' else c_led_off;
  led_state(15 downto 14) <= c_led_green when fp_gpio_led(1) = '1' else c_led_off;

  -- Front panel IO configuration
  fp_term_en_o    <= (others => '0');
  fp_gpio1_a2b_o  <= '0';
  fp_gpio2_a2b_o  <= '0';
  fp_gpio34_a2b_o <= '0';

  fp_gpio_in(1) <= fp_gpio1_b;
  fp_gpio_in(2) <= fp_gpio2_b;
  fp_gpio_in(3) <= fp_gpio3_b;
  fp_gpio_in(4) <= fp_gpio4_b;

  -- Front panel IO synchronisation
  gen_fp_gpio_sync: for i in 1 to 4 generate
    begin
      cmp_fp_gpio_sync: gc_sync_ffs
        port map (
          clk_i    => clk_sys_62m5,
          rst_n_i  => '1',
          data_i   => fp_gpio_in(i),
          ppulse_o => fp_gpio(i)
        );

      cmp_fp_gpio_led : gc_extend_pulse
        generic map (
          g_width => 2500000
        )
        port map (
          clk_i      => clk_sys_62m5,
          rst_n_i    => rst_sys_62m5_n,
          pulse_i    => fp_gpio(i),
          extended_o => fp_gpio_led(i)
        );
    end generate;

  -----------------------------------------------------------------------------
  -- FMC slot 0 - FFPG
  -----------------------------------------------------------------------------

  cFmc0ClkIn0Ibufgds: IBUFGDS
    generic map (
      DIFF_TERM => TRUE
    )
    port map (
      I   => Fmc0ClkIn0P_ik,
      IB  => Fmc0ClkIn0N_ik,
      O   => Fmc0ClkIn0_k
    );

  cFmc0TriggerIbufds: IBUFDS
    generic map (
      DIFF_TERM => TRUE
    )
    port map (
      I   => Fmc0TriggerP_i,
      IB  => Fmc0TriggerN_i,
      O   => Fmc0Trigger
    );

  cFmc0FfpgCore: entity work.FfpgCore(syn)
    generic map (
      g_ClkFrequency    => 62500000   -- in Hz
    )
    port map (
      -- Wishbone connection
      Clk_ik            => clk_sys_62m5,
      Reset_ira         => not rst_sys_62m5_n,
      Wb_i              => ffpg0_wb_in,
      Wb_o              => ffpg0_wb_out,
      ---- FMC interface
      -- clock
      ClkIn0_ik         => Fmc0ClkIn0_k,
      -- DACs
      TriggerDac_o      => Fmc0TriggerDac_o,
      VcxoDac_o         => Fmc0VcxoDac_o,
      -- output enable
      Ch1OutputEnable_o => Fmc0Ch1OutputEnable_o,
      Ch2OutputEnable_o => Fmc0Ch2OutputEnable_o,
      -- delay configuration
      DelayValue_ob     => Fmc0DelayValue_ob,
      Ch1SetLe_on       => Fmc0Ch1SetLe_on,
      Ch1ResLe_on       => Fmc0Ch1ResLe_on,
      Ch2SetLe_on       => Fmc0Ch2SetLe_on,
      Ch2ResLe_on       => Fmc0Ch2ResLe_on,
      Ch1Set_o          => Fmc0Ch1Set,
      Ch1Res_o          => Fmc0Ch1Res,
      Ch2Set_o          => Fmc0Ch2Set,
      Ch2Res_o          => Fmc0Ch2Res,
      Trigger_i         => Fmc0Trigger,
      -- AD9512 SPI
      SpiAd9512Sclk_o   => Fmc0SpiAd9512Sclk_o,
      SpiAd9512Mosi_o   => Fmc0SpiAd9512Mosi_o,
      SpiAd9512Miso_i   => Fmc0SpiAd9512Miso_i,
      SpiAd9512Cs_on    => Fmc0SpiAd9512Cs_on,
      -- AD9512 func. pin
      Ad9512Func_o      => Fmc0Ad9512Func_o,
      -- clock selection pin of SY58017U
      Clk2Sel_o         => Fmc0Clk2Sel_o,
      -- clock output
      ClkOut_ok         => Fmc0ClkOut_k,
      -- temperature chip interface
      Onewire_io        => Fmc0Onewire_io,
      -- LEDs
      Led_ob            => Fmc0Led_ob
    );

  cFmc0Ch1SetObufds: OBUFDS
    port map (
      I   => Fmc0Ch1Set,
      O   => Fmc0Ch1SetP_o,
      OB  => Fmc0Ch1SetN_o
    );

  cFmc0Ch1ResObufds: OBUFDS
    port map (
      I   => Fmc0Ch1Res,
      O   => Fmc0Ch1ResP_o,
      OB  => Fmc0Ch1ResN_o
    );

  cFmc0Ch2SetObufds: OBUFDS
    port map (
      I   => Fmc0Ch2Set,
      O   => Fmc0Ch2SetP_o,
      OB  => Fmc0Ch2SetN_o
    );

  cFmc0Ch2ResObufds: OBUFDS
    port map (
      I   => Fmc0Ch2Res,
      O   => Fmc0Ch2ResP_o,
      OB  => Fmc0Ch2ResN_o
    );

  cFmc0ClkOutOddr: ODDR2
    generic map (
      DDR_ALIGNMENT => "C0",
      SRTYPE        => "ASYNC"
    )
    port map (
      Q   => Fmc0ClkOutDdr,
      C0  => Fmc0ClkOut_k,
      C1  => not Fmc0ClkOut_k,
      CE  => '1',
      D0  => '1',
      D1  => '0',
      R   => '0',
      S   => '0'
    );

  cFmc0ClkOutObufds: OBUFDS
    port map (
      I   => Fmc0ClkOutDdr,
      O   => Fmc0ClkOutP_ok,
      OB  => Fmc0ClkOutN_ok
    );

  -----------------------------------------------------------------------------
  -- FMC slot 1 - FFPG
  -----------------------------------------------------------------------------

  cFmc1ClkIn0Ibufgds: IBUFGDS
    generic map (
      DIFF_TERM => TRUE
    )
    port map (
      I   => Fmc1ClkIn0P_ik,
      IB  => Fmc1ClkIn0N_ik,
      O   => Fmc1ClkIn0_k
    );

  cFmc1TriggerIbufds: IBUFDS
    generic map (
      DIFF_TERM => TRUE
    )
    port map (
      I   => Fmc1TriggerP_i,
      IB  => Fmc1TriggerN_i,
      O   => Fmc1Trigger
    );

  cFmc1FfpgCore: entity work.FfpgCore(syn)
    generic map (
      g_ClkFrequency    => 62500000   -- in Hz
    )
    port map (
      -- Wishbone connection
      Clk_ik            => clk_sys_62m5,
      Reset_ira         => not rst_sys_62m5_n,
      Wb_i              => ffpg1_wb_in,
      Wb_o              => ffpg1_wb_out,
      ---- FMC interface
      -- clock
      ClkIn0_ik         => Fmc1ClkIn0_k,
      -- DACs
      TriggerDac_o      => Fmc1TriggerDac_o,
      VcxoDac_o         => Fmc1VcxoDac_o,
      -- output enable
      Ch1OutputEnable_o => Fmc1Ch1OutputEnable_o,
      Ch2OutputEnable_o => Fmc1Ch2OutputEnable_o,
      -- delay configuration
      DelayValue_ob     => Fmc1DelayValue_ob,
      Ch1SetLe_on       => Fmc1Ch1SetLe_on,
      Ch1ResLe_on       => Fmc1Ch1ResLe_on,
      Ch2SetLe_on       => Fmc1Ch2SetLe_on,
      Ch2ResLe_on       => Fmc1Ch2ResLe_on,
      Ch1Set_o          => Fmc1Ch1Set,
      Ch1Res_o          => Fmc1Ch1Res,
      Ch2Set_o          => Fmc1Ch2Set,
      Ch2Res_o          => Fmc1Ch2Res,
      Trigger_i         => Fmc1Trigger,
      -- AD9512 SPI
      SpiAd9512Sclk_o   => Fmc1SpiAd9512Sclk_o,
      SpiAd9512Mosi_o   => Fmc1SpiAd9512Mosi_o,
      SpiAd9512Miso_i   => Fmc1SpiAd9512Miso_i,
      SpiAd9512Cs_on    => Fmc1SpiAd9512Cs_on,
      -- AD9512 func. pin
      Ad9512Func_o      => Fmc1Ad9512Func_o,
      -- clock selection pin of SY58017U
      Clk2Sel_o         => Fmc1Clk2Sel_o,
      -- clock output
      ClkOut_ok         => Fmc1ClkOut_k,
      -- temperature chip interface
      Onewire_io        => Fmc1Onewire_io,
      -- LEDs
      Led_ob            => Fmc1Led_ob
    );

  cFmc1Ch1SetObufds: OBUFDS
    port map (
      I   => Fmc1Ch1Set,
      O   => Fmc1Ch1SetP_o,
      OB  => Fmc1Ch1SetN_o
    );

  cFmc1Ch1ResObufds: OBUFDS
    port map (
      I   => Fmc1Ch1Res,
      O   => Fmc1Ch1ResP_o,
      OB  => Fmc1Ch1ResN_o
    );

  cFmc1Ch2SetObufds: OBUFDS
    port map (
      I   => Fmc1Ch2Set,
      O   => Fmc1Ch2SetP_o,
      OB  => Fmc1Ch2SetN_o
    );

  cFmc1Ch2ResObufds: OBUFDS
    port map (
      I   => Fmc1Ch2Res,
      O   => Fmc1Ch2ResP_o,
      OB  => Fmc1Ch2ResN_o
    );

  cFmc1ClkOutOddr: ODDR2
    generic map (
      DDR_ALIGNMENT => "C0",
      SRTYPE        => "ASYNC"
    )
    port map (
      Q   => Fmc1ClkOutDdr,
      C0  => Fmc1ClkOut_k,
      C1  => not Fmc1ClkOut_k,
      CE  => '1',
      D0  => '1',
      D1  => '0',
      R   => '0',
      S   => '0'
    );

  cFmc1ClkOutObufds: OBUFDS
    port map (
      I   => Fmc1ClkOutDdr,
      O   => Fmc1ClkOutP_ok,
      OB  => Fmc1ClkOutN_ok
    );

end architecture top;
